#ifndef Application_h
#define Application_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "BaseCoreApplication.h"
#include "CommandLineArguments.h"
#include "Config.h"
#include "Counter.h"

//* display application manager server messages
class CoreApplication: public BaseCoreApplication, private Base::Counter<CoreApplication>
{

    //* Qt meta object declaration
    Q_OBJECT

    public:

    //* constructor
    explicit CoreApplication( CommandLineArguments );

    //* destructor
    ~CoreApplication() override;

    //* initialize application manager
    bool initApplicationManager() override;

    //*@name application information
    //@{

    //* command line parser
    CommandLineParser commandLineParser( CommandLineArguments = CommandLineArguments(), bool ignoreWarnings = true ) const override;

    //* usage
    void usage() const override;

    //* application name
    QString applicationName() const override
    { return "ServerCommand"; }

    // application version
    QString applicationVersion() const override
    { return VERSION; }

    //@}

    public Q_SLOTS:

    //* server command
    virtual void sendServerCommand( Server::ServerCommand ) override;

    //* server command
    virtual void requestIdentify();

    protected Q_SLOTS:

    //* process request from application manager
    virtual bool _processCommand( Server::ServerCommand ) override;

    private Q_SLOTS:

    //* connect client
    void _connectClient();

    //* append message to the log
    void _append( Server::ServerCommand );


};

#endif
