#ifndef MainWindow_h
#define MainWindow_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "BaseMainWindow.h"
#include "Debug.h"
#include "ServerCommand.h"
#include "ServerCommandModel.h"

#include <QAction>
#include <QCloseEvent>

class CustomToolBar;
class TreeView;
class BaseStatusBar;

/*!
\class MainWindow
\brief customized QVBox to recieve close event
*/
class MainWindow: public BaseMainWindow
{

    //* Qt meta object declaration
    Q_OBJECT

    public:

    //* constructor
    explicit MainWindow( QWidget* parent = 0 );

    //* verbosity enumeration
    enum Verbosity
    {
        VerbosityLow,
        VerbosityHigh
    };

    //* Verbosity
    void setVerbosity( Verbosity verbosity )
    { verbosity_ = verbosity; }

    //* statusBar
    BaseStatusBar& statusBar()
    { return *statusBar_; }

    //* toolbar
    CustomToolBar& toolBar() const
    { return *toolbar_; }

    //* append command to list
    void append( const ServerCommandModel::List& );

    //*@name actions
    //@{

    //* reset action
    QAction& resetAction() const
    { return *resetAction_; }

    //* unlock action
    QAction& unlockAction() const
    { return *unlockAction_; }

    //* raise action
    QAction& raiseAction() const
    { return *raiseAction_; }

    //* raise action
    QAction& abortAction() const
    { return *abortAction_; }

    //@}

    Q_SIGNALS:

    //* request command to be passed to application application manager
    /*! the application ID corresponds to this application */
    void requestCommand( Server::ServerCommand::CommandType );

    //* request command to be passed to application application manager
    void requestCommand( Server::ServerCommand );

    protected:

    //* close window event handler
    void closeEvent( QCloseEvent *event );

    private Q_SLOTS:

    //* configuration
    void _updateConfiguration();

    //* update actions
    void _updateActions();

    //* request reset command
    void _requestReset();

    //* request unlock command
    void _requestUnlock();

    //* request raise command
    void _requestRaise();

    //* request abort command
    void _requestAbort();

    private:

    //* actions
    void _installActions();

    //* verbosity
    Verbosity verbosity_ = VerbosityLow;

    //* Command list
    TreeView* treeView_ = nullptr;

    //* command model
    ServerCommandModel model_;

    //* toolbar
    CustomToolBar* toolbar_ = nullptr;

    //* state frame
    BaseStatusBar* statusBar_ = nullptr;

    //*@name actions
    //@{

    //* reset
    QAction* resetAction_ = nullptr;

    //* unlock
    QAction* unlockAction_ = nullptr;

    //* raise application action
    QAction* raiseAction_ = nullptr;

    //* abort application action
    QAction* abortAction_ = nullptr;

    //@}

};

#endif
