#ifndef ServerCommandModel_h
#define ServerCommandModel_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "ServerCommand.h"
#include "TreeModel.h"

#include <QColor>
#include <QIcon>
#include <QHash>

#include <array>

//* ServerCommand model. Stores job information for display in lists
class ServerCommandModel : public TreeModel<Server::ServerCommand>, private Base::Counter<ServerCommandModel>
{

    //* Qt meta object declaration
    Q_OBJECT

    public:

    //* column type enumeration
    enum ColumnTypes
    {
        Name,
        User,
        Display,
        Command,
        Pid,
        Time,
        nColumns
    };

    //* constructor
    explicit ServerCommandModel( QObject* = nullptr );

    //*@name methods reimplemented from base class
    //@{

    //* return data
    QVariant data( const QModelIndex&, int ) const override;

    //* header data
    QVariant headerData( int, Qt::Orientation, int = Qt::DisplayRole ) const override;

    //* number of columns for a given index
    int columnCount( const QModelIndex& = QModelIndex() ) const override
    { return nColumns; }

    //@}

    //* remove
    void remove( ServerCommandModel::List );

    //* clear
    void clear();

    protected:

    //* get icon for pid
    QIcon _icon( qint64 ) const;

    //* sorting
    void _sort( int column, Qt::SortOrder order ) override
    { _root().sort( SortFTor( (ColumnTypes) column, order ) ); }

    private Q_SLOTS:

    //* configuration
    void _updateConfiguration();

    private:

    //* used to sort IconCaches
    class SortFTor: public ItemModel::SortFTor
    {

        public:

        //* constructor
        explicit SortFTor( int type, Qt::SortOrder order ):
            ItemModel::SortFTor( type, order )
        {}

        //* prediction
        bool operator() ( const Item& first, const Item& second ) const
        { return (*this)( first.get(), second.get() ); }

        //* prediction
        bool operator() ( Server::ServerCommand, Server::ServerCommand ) const;

    };

    //* column titles
    const std::array<QString, nColumns> columnTitles_ =
    {{
        tr( "Name" ),
        tr( "User" ),
        tr( "Display" ),
        tr( "Command" ),
        tr( "Process Id" ),
        tr( "Time" )
    }};

    //* colors
    QColor identifyColor_;

    //* colors
    QColor serverColor_;

    //* true if icons must be shown
    bool showIcons_;

    //* icon size
    QSize iconSize_;

    //* empty icon
    QIcon emptyIcon_;

    //* map icon and pid
    using IconMap = QHash<qint64, QIcon>;

    //* map icon and pid
    IconMap icons_;

};

#endif
