/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "XcbInterface.h"

#include "Debug.h"
#include "XcbUtil.h"

#if WITH_XCB
#include <xcb/xcb.h>
#endif

//_________________________________________________________
QIcon XcbInterface::findIcon( qint64 pid ) const
{

    // check atoms
    #if WITH_XCB
    if( !( XcbUtil::get().isSupported( XcbDefines::_NET_WM_PID ) && XcbUtil::get().isSupported( XcbDefines::_NET_WM_ICON ) ) )
    {

        return QIcon();

    } else return _findIcon( XcbUtil::get().appRootWindow(), pid );
    #else
    return QIcon();
    #endif
}

//_________________________________________________________
QIcon XcbInterface::_findIcon( WId window, qint64 pid ) const
{

    #if WITH_XCB
    Debug::Throw() << "XcbInterface::_findIcons - window: " << window << endl;

    if( !XcbUtil::isX11() ) return QIcon();

    // connection and atom
    auto connection( XcbUtil::get().connection<xcb_connection_t>() );
    auto atom( *XcbUtil::get().atom<xcb_atom_t>( XcbDefines::_NET_WM_PID ) );

    xcb_get_property_cookie_t cookie( xcb_get_property( connection, 0, window, atom, XCB_ATOM_CARDINAL, 0, 1 ) );
    XcbUtil::ScopedPointer<xcb_get_property_reply_t> reply( xcb_get_property_reply( connection, cookie, nullptr ) );
    if( reply )
    {
        const uint32_t windowPid( reinterpret_cast<uint32_t*>(xcb_get_property_value( reply.get() ))[0] );
        if( windowPid == pid )
        {
            // find window icon
            QIcon icon( XcbUtil::get().icon( window ) );
            if( !icon.isNull() ) return icon;
        }
    }

    {
        // parse children
        xcb_query_tree_cookie_t cookie = xcb_query_tree( connection, window );
        XcbUtil::ScopedPointer<xcb_query_tree_reply_t> reply( xcb_query_tree_reply( connection, cookie, nullptr ) );
        if( !reply ) return QIcon();

        const int childCount( xcb_query_tree_children_length( reply.get() ) );
        xcb_window_t* children( xcb_query_tree_children( reply.get() ) );
        for( int i=0; i < childCount; ++i )
        {
            QIcon icon = _findIcon( children[i], pid );
            if( !icon.isNull() ) return icon;
        }
    }

    #endif
    return QIcon();

}
