/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CoreApplication.h"
#include "Debug.h"
#include "DefaultOptions.h"
#include "ResourceMigration.h"
#include "Singleton.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QCoreApplication>

//_______________________________
//! to handle keyboard interuptions
void interupt( int sig );

//__________________________________________
int main (int argc, char *argv[])
{

    // options
    installDefaultOptions();

    // migrate old rc files
    File oldRCFile( XmlOptions::get().raw( "OLD_RC_FILE" ) );
    File rcFile( XmlOptions::get().raw( "RC_FILE" ) );
    ResourceMigration( oldRCFile ).migrate( rcFile );

    // assign and read
    XmlOptions::setFile( rcFile );
    XmlOptions::read();

    // debug level
    Debug::setLevel( XmlOptions::get().get<int>( "DEBUG_LEVEL" ) );
    if( Debug::level() ) Debug::Throw() << XmlOptions::get() << endl;

    QCoreApplication application( argc, argv );

    const CommandLineArguments arguments( argc, argv );
    CoreApplication singleton( arguments );
    Base::Singleton::get().setApplication( &singleton );

    singleton.initApplicationManager();

    // parse arguments
    CommandLineParser parser( singleton.commandLineParser( arguments ) );
    if( parser.hasOption( "--send" ) )
    {

        QString command( parser.option( "--send" ) );
        Debug::Throw() << "main - command: " << command << endl;
        CommandLineArguments commandArguments( command.split( QRegExp( "\\s+" ) ) );

        // build Application Id
        Server::ApplicationId id( commandArguments[0] );
        Server::ServerCommand serverCommand( id, Server::ServerCommand::Request );
        serverCommand.setArguments( commandArguments );
        singleton.sendServerCommand( serverCommand );

    }

    if( parser.hasFlag( "--list" ) )
    { singleton.requestIdentify(); }

    return 0;
}
